/******************************************************************************
 GMailboxTreeDir.cc

	<Description>

	BASE CLASS = public GMManagedDirector

	Copyright  1997 by Glenn W. Bach.  All rights reserved.
	
	Base code generated by Codemill v0.1.0

 *****************************************************************************/

// includes

//Class Header
#include <GMailboxTreeDir.h>
#include <GSimpleFileTree.h>
#include <JNamedTreeNode.h>
#include <GMailboxTreeWidget.h>
#include <JNamedTreeList.h>
#include <JXNamedTreeListWidget.h>
#include <GMGlobals.h>
#include <GPrefsMgr.h>
#include "GHelpText.h"
#include <GMApp.h>
#include "GMDirectorManager.h"
#include "GMDirectorMenu.h"

#include "mailbox.xpm"
#include "jx_help_toc.xpm"
#include "jx_help_specific.xpm"

#include <JXToolBar.h>

#include <JChooseSaveFile.h>
#include <JXWindow.h>
#include <JXTextMenu.h>
#include <JXMenuBar.h>
#include <JXScrollbarSet.h>
#include <JXApplication.h>
#include <JXTextButton.h>
#include <JXTimerTask.h>
#include <JXHelpManager.h>
#include <JXImage.h>

#include <JString.h>

#include <jAssert.h>

static const JCharacter* kWindowsMenuTitleStr = "Windows";

static const JCharacter* kHelpMenuTitleStr = "Help";
static const JCharacter* kHelpMenuStr = 
	"About"
	"%l|Table of Contents %i \"TOCHelp::Arrow\""
	"|Overview|This window %i \"ThisWindowHelp::Arrow\""
	"%l|Changes|Credits";

enum
{	
	kAboutCmd = 1,
	kTOCCmd,
	kOverviewCmd,
	kThisWindowCmd,
	kChangesCmd,
	kCreditsCmd
};

const JCoordinate kCloseButtonWidth 	= 50;
const JCoordinate kCurrentPrefsVersion	= 2;

const Time kDirUpdateDelay = 1000;	// milliseconds

const JCharacter* kGMMailboxDirMenuShortcut	= "Meta-Shift-M";
const JCharacter* kGMMailboxDirMenuID		= "MailboxWindow::Arrow";

/******************************************************************************
 Constructor

 *****************************************************************************/

GMailboxTreeDir::GMailboxTreeDir
	(
	JXDirector* supervisor
	)
	:
   GMManagedDirector(supervisor),
   JPrefObject(GGetPrefsMgr(), kMailTreePrefsID),
   itsTimerTask(NULL)
{
	BuildWindow();
	JPrefObject::ReadPrefs();
	itsWidget->Update();

	GGetDirMgr()->DirectorCreated(this);
}

/******************************************************************************
 Destructor

 *****************************************************************************/

GMailboxTreeDir::~GMailboxTreeDir()
{
	JPrefObject::WritePrefs();
	delete itsTree;
	delete itsMenuIcon;
	GGetDirMgr()->DirectorDeleted(this);
}

/******************************************************************************
 BuildWindow 


 ******************************************************************************/

void
GMailboxTreeDir::BuildWindow()
{
	JCoordinate w = 300;
	JCoordinate h = 500;
	JXWindow* window = new JXWindow(this, w,h, "Mailbox window");
    assert( window != NULL );
    SetWindow(window);
	window->SetWMClass(GMGetWMClassInstance(), GMGetMailboxWindowClass());
	window->SetCloseAction(JXWindow::kDeactivateDirector);
    window->PlaceAsDialogWindow();
    window->SetFocusWhenShow(kTrue);
    window->LockCurrentMinSize();
    
	JXMenuBar* menuBar =
		new JXMenuBar(window, JXWidget::kHElastic, JXWidget::kFixedTop, 
			0,0, w - kCloseButtonWidth,kJXStdMenuBarHeight);
	assert( menuBar != NULL );

    itsCloseButton =
        new JXTextButton("Close", window,
                    JXWidget::kFixedRight, JXWidget::kFixedTop, 
                    w - kCloseButtonWidth,0, 
                    kCloseButtonWidth,kJXStdMenuBarHeight);
    assert( itsCloseButton != NULL );
    itsCloseButton->SetShortcuts("#W"); //^[
    ListenTo(itsCloseButton);

	itsToolBar = 
		new JXToolBar(GGetPrefsMgr(), kGMailboxToolBarID,
			menuBar, w, h, window,
			JXWidget::kHElastic, JXWidget::kVElastic, 
			0, kJXStdMenuBarHeight, w, h - kJXStdMenuBarHeight);
	assert(itsToolBar != NULL);

	const JSize newHeight = itsToolBar->GetWidgetEnclosure()->GetBoundsHeight();
	
	JXScrollbarSet* scrollbarSet =
		new JXScrollbarSet(itsToolBar->GetWidgetEnclosure(), 
						   JXWidget::kHElastic,JXWidget::kVElastic,
						   0, 0,
						   w, newHeight);
	assert( scrollbarSet != NULL );
    
    JNamedTreeNode* base = new JNamedTreeNode(NULL, "");
    assert(base != NULL);
    itsTree = new GSimpleFileTree(base);
    assert(itsTree != NULL);
    JNamedTreeList* list = new JNamedTreeList(itsTree);
    itsTree->SetTreeList(list);
    
    itsWidget = 
    	GMailboxTreeWidget::Create(itsTree, list,  menuBar,
    		scrollbarSet, scrollbarSet->GetScrollEnclosure(),
    		JXWidget::kHElastic, JXWidget::kVElastic,
    		0,0,w,newHeight);
    assert(itsWidget != NULL);
    
	GMDirectorMenu* menu = 
		new GMDirectorMenu(kWindowsMenuTitleStr, menuBar,
			JXWidget::kFixedLeft, JXWidget::kVElastic, 0,0, 10,10);
	assert(menu != NULL);
	menuBar->AppendMenu(menu);

	itsHelpMenu = menuBar->AppendTextMenu(kHelpMenuTitleStr);
	itsHelpMenu->SetMenuItems(kHelpMenuStr);
	itsHelpMenu->SetUpdateAction(JXMenu::kDisableNone);
	ListenTo(itsHelpMenu);

	itsHelpMenu->SetItemImage(kTOCCmd, JXPM(jx_help_toc));
	itsHelpMenu->SetItemImage(kThisWindowCmd, JXPM(jx_help_specific));

	itsToolBar->LoadPrefs();

	if (itsToolBar->IsEmpty())
		{
		itsWidget->AddDefaultButtcons(itsToolBar);
		itsToolBar->AppendButton(itsHelpMenu, kTOCCmd);
		itsToolBar->AppendButton(itsHelpMenu, kThisWindowCmd);
		}

	itsMenuIcon = new JXImage(window->GetDisplay(), window->GetColormap(), JXPM(mailbox));
	assert(itsMenuIcon != NULL);
	itsMenuIcon->ConvertToRemoteStorage();

}

/******************************************************************************
 Receive (virtual protected)


 ******************************************************************************/

void
GMailboxTreeDir::Receive
	(
	JBroadcaster*	sender,
	const Message&	message
	)
{
	if (sender == itsCloseButton && message.Is(JXButton::kPushed))
		{
		Deactivate();
		}
	else if (sender == itsTimerTask && message.Is(JXTimerTask::kTimerWentOff))
		{
		itsWidget->Update();
		}
	else if (sender == itsHelpMenu && message.Is(JXMenu::kItemSelected))
		{
		 const JXMenu::ItemSelected* selection =
			dynamic_cast(const JXMenu::ItemSelected*, &message);
		assert( selection != NULL );
		HandleHelpMenu(selection->GetIndex());
		}
	else
		{
		GMManagedDirector::Receive(sender, message);
		}
}

/******************************************************************************
 ReadPrefs (protected)

 ******************************************************************************/

void
GMailboxTreeDir::ReadPrefs
	(
	istream& input
	)
{
	JCoordinate id;
	input >> id;
	assert(id <= kCurrentPrefsVersion);
	GetWindow()->ReadGeometry(input);
	if (id > 0)
		{
		itsWidget->ReadPrefs(input);
		}
	if (id > 1)
		{
		JBoolean active;
		input >> active;
		JBoolean iconified;
		input >> iconified;
		if (iconified)
			{
			GetWindow()->Deiconify();
			}
		if (active && !iconified)
			{
			Activate();
			}
		}
}

/******************************************************************************
 WritePrefs (protected)

 ******************************************************************************/

void
GMailboxTreeDir::WritePrefs
	(
	ostream& output
	)
	const
{
	output << kCurrentPrefsVersion << ' ';
	GetWindow()->WriteGeometry(output);
	itsWidget->WritePrefs(output);
	output << IsActive() << ' ';
	output << GetWindow()->IsIconified() << ' ';
}

/******************************************************************************
 HandleHelpMenu 


 ******************************************************************************/

void
GMailboxTreeDir::HandleHelpMenu
	(
	const JIndex index
	)
{
	if (index == kAboutCmd)
		{
		GMGetApplication()->DisplayAbout();
		}
	else if (index == kTOCCmd)
		{
		(JXGetHelpManager())->ShowSection(kTOCHelpName);
		}
	else if (index == kOverviewCmd)
		{
		(JXGetHelpManager())->ShowSection(kOverviewHelpName);
		}
	else if (index == kThisWindowCmd)
		{
		(JXGetHelpManager())->ShowSection(kMailTreeHelpName);
		}
	else if (index == kChangesCmd)
		{
		(JXGetHelpManager())->ShowSection(kChangeLogName);
		}
	else if (index == kCreditsCmd)
		{
		(JXGetHelpManager())->ShowSection(kCreditsName);
		}

}

/******************************************************************************
 EditToolBar (public)

 ******************************************************************************/

void
GMailboxTreeDir::EditToolBar()
{
	itsToolBar->Edit();
}

/******************************************************************************
 GetShortcut (public)

 ******************************************************************************/

JBoolean
GMailboxTreeDir::GetShortcut
	(
	JString* shortcut
	)
	const
{
	*shortcut = kGMMailboxDirMenuShortcut;
	return kTrue;
}

/******************************************************************************
 GetID (public)

 ******************************************************************************/

JBoolean
GMailboxTreeDir::GetID
	(
	JString* id
	)
	const
{
	*id = kGMMailboxDirMenuID;
	return kTrue;
}

/******************************************************************************
 Activate (virtual protected)

 ******************************************************************************/

void
GMailboxTreeDir::Activate()
{
	if (itsTimerTask == NULL)
		{
		itsTimerTask = new JXTimerTask(kDirUpdateDelay);
		assert(itsTimerTask != NULL);
		}

	JXGetApplication()->InstallIdleTask(itsTimerTask);
	ListenTo(itsTimerTask);

	GMManagedDirector::Activate();
}

/******************************************************************************
 Deactivate (virtual protected)

 ******************************************************************************/

JBoolean
GMailboxTreeDir::Deactivate()
{
	if (itsTimerTask != NULL)
		{
		JXGetApplication()->RemoveIdleTask(itsTimerTask);
		}
		
	GMManagedDirector::Deactivate();
}
